#!/usr/bin/env python2

import unittest, md5, random, socket, time, os, re, tempfile

class TestBase:

        def __init__(self):
            try:
                rgmaHome = os.environ['RGMA_HOME']
            except KeyError:
                raise "ERROR: Environment variable RGMA_HOME is not set"
            
            self.rgmaCommand = os.path.join(rgmaHome, "bin/rgma")
            self.createUniqueId()
            
        def createUniqueId(self, args=None):
            """
            Create a unique ID to insert into userTable. This ensures that
            tests work if there is other data in the table
            """
            t = long( time.time() * 1000 )
            r = long( random.random()*100000000000000000L )
            try:
                a = socket.gethostbyname( socket.gethostname() )
            except:
                # if we can't get a network address, just imagine one
                a = random.random()*100000000000000000L
            data = str(t)+' '+str(r)+' '+str(a)+' '+ str(args)
            self.uniqueId = md5.md5(data).hexdigest()

        def getOutput(self, args):
            """
            Run the RGMA command line tool with the specified arguments and
            return the output as a list of strings
            """
            command = "%s %s" % (self.rgmaCommand, args)
            (stdin, stdout, stderr) = os.popen3(command)
            output = stdout.readlines()
	    stdout.close()
	    stderr.close()
	    stdin.close()
            return output

        def getErrorOutput(self, args):
            """
            Run the RGMA command line tool with the specified arguments and
            return the output as a list of strings
            """
            command = "%s %s" % (self.rgmaCommand, args)
            (stdin, stdout, stderr) = os.popen3(command)
            error = stderr.readlines()
	    stdout.close()
	    stderr.close()
	    stdin.close()
            return error

        def getStatus(self, args):
            """
            Run the RGMA command line tool with the specified arguments and
            return the output as a list of strings
            """
            command = "%s %s 2>/dev/null" % (self.rgmaCommand, args)
            stdout = os.popen(command)
	    status = stdout.close()
	    if status != None:
                return status >> 8
    	    else:
	        return 0
	
        def checkOutput(self, output, expected):
	    """
	    Check that the regular expression <expected> matches a line in <output>
	    """
            found = False
            for line in output:
                if re.match(expected, line.strip()):
                    found = True
            assert found
                
        def checkOutputNotPresent(self, output, expected):
	    """
	    Check that the regular expression <expected> does not match any line in <output>
	    """
            found = False
            for line in output:
                if re.match(expected, line.strip()):
                    found = True
            assert not found
                
        def checkNoTuples(self, output):
            """
            Check that no tuples were returned by a select query
            """
            self.checkOutput(output, "^No results returned")
            
        def checkOneTuple(self, output):
            """
            Check that one tuple was returned by a select query
            """
            self.checkOutput(output, "^1 rows")
            
        def checkRightTuple(self, output):
            """
            Check that a tuple matching our unique user ID and data was 
            returned by a select query
            """
            regex = "\|\s*%s\s*\|\s*a\s*\|\s*1\.2\s*\|\s*3\s*\|\s*[\d-]+\s*\|\s*[\d:]+\s*\|\s*" % self.uniqueId
            self.checkOutput(output, regex)
            
        def continuousInsert(self):
            """
            Insert a tuple using a continuous primary producer
            """
            args = """-c "set producer continuous" \
                      -c "insert into userTable (userId, aString, aReal, anInt) values ('%s', 'a', 1.2, 3)" \
                   """ % self.uniqueId
            return self.getOutput(args)
            
        def latestInsert(self):
            """
            Insert a tuple using a continuous primary producer
            """
            args = """-c "set producer latest" \
                      -c "insert into userTable (userId, aString, aReal, anInt) values ('%s', 'a', 1.2, 3)" \
                   """ % self.uniqueId
            return self.getOutput(args)
            
        def continuousSelect(self):
            """
            Run a continuous select query
            """
            args = """-c "set query continuous" \
                      -c "set timeout 5 seconds" \
                      -c "set maxage None" \
                      -c "select * from userTable where userId='%s'" \
                   """ % self.uniqueId
            return self.getOutput(args)
 
        def continuousOldSelect(self):
            """
            Run a continuous select query that also returns old tuples (up
            to one hour old)
            """
            args = """-c "set query continuous" \
                      -c "set timeout 5 seconds" \
                      -c "set maxage 1 hours" \
                      -c "select * from userTable where userId='%s'" \
                   """ % self.uniqueId
            return self.getOutput(args)

        def latestSelect(self):
            """
            Run a continuous select query that also returns old tuples (up
            to one hour old)
            """
            args = """-c "set query latest" \
                      -c "set timeout 5 seconds" \
                      -c "set maxage none" \
                      -c "select * from userTable where userId='%s'" \
                   """ % self.uniqueId
            return self.getOutput(args)


class CommandLineArgumentTests(unittest.TestCase, TestBase):
                              
        def setUp(self):
            TestBase.__init__(self)
        
        def suite(self):
            return unittest.makeSuite(CommandLineArgumentTests, 'test')
            
        def testBadArgument(self):
	    """
	    Test exit status of 1 for an unrecognized command line switch
	    """
	    status = self.getStatus("--junk")
	    assert(status == 1)
	    
        def testVersion(self):
	    """
	    Test --version produces a sensible version number
	    """
            output = self.getOutput("--version")
            self.checkOutput(output, "R-GMA command line tool version \d\.\d\.\d")
            
        def testV(self):
	    """
	    Test -v produces a sensible version number
	    """
            output = self.getOutput("-v")
            self.checkOutput(output, "R-GMA command line tool version \d\.\d\.\d")

        def testHelp(self):
	    """
	    Test --help produces something sensible
	    """
            output = self.getOutput("--help")
            self.checkOutput(output, "Usage")
            
        def testH(self):
	    """
	    Test -h produces something sensible
	    """
            output = self.getOutput("-h")
            self.checkOutput(output, "Usage")

        def testCommand(self):
	    """
	    Test the use of -c <command>
	    """
            output = self.getOutput("-c exit")
            self.checkOutput(output, "Goodbye")

        def testBatchFile(self):
	    """
	    Test the use of -f <file>
	    """
            tempName = tempfile.mktemp(".rgma")
            try:
                tempFile = open(tempName, "w")
                tempFile.write("set timeout 0.1 seconds\n")
                tempFile.write("exit\n")
                tempFile.close()         
                output = self.getOutput("-f %s" % tempName)
                self.checkOutput(output, "Set timeout to 0.1.* seconds")
                self.checkOutput(output, "Goodbye")
            finally:
                os.remove(tempName)    
                        
class InsertSelectTests(unittest.TestCase, TestBase):
	
	def setUp(self):
	    TestBase.__init__(self)
                		
	def suite(self):
	    return unittest.makeSuite(InsertSelectTests, 'test')

	def testInsertContinuousContinuousSelect(self):
	    """
	    Check that a tuple inserted into a continuous producer is not returned by a subsequent continuous query 
	    """
            self.continuousInsert()
            output = self.continuousSelect()
            self.checkNoTuples(output)
        
        def testInsertContinuousOldSelect(self):
	    """
	    Check that a tuple inserted into a continuous producer is returned by a continuous+old query 
	    """
            self.continuousInsert()
            output = self.continuousOldSelect()
            self.checkOneTuple(output)
            self.checkRightTuple(output)

	def testInsertLatestContinuousSelect(self):
	    """
	    Check that a tuple inserted into a latest producer is not returned by a continuous query 
	    """
            self.latestInsert()
            output = self.continuousSelect()
            self.checkNoTuples(output)
        
        def testInsertLatestOldSelect(self):
	    """
	    Check that a tuple inserted into a latest producer is returned by a continuous+old query 
	    """
            self.latestInsert()
            output = self.continuousOldSelect()
            self.checkOneTuple(output)
            self.checkRightTuple(output)

        def testInsertLatestLatestSelect(self):
	    """
	    Check that a tuple inserted into a latest producer is returned by a latest query 
	    """
            args = """-c "set producer latest" \
                      -c "insert into userTable (userId, aString, aReal, anInt) values ('%s', 'a', 1.2, 3)" \
                   """ % (self.uniqueId)
            status = self.getStatus(args)
	    assert(status == 0)

	    time.sleep(30)
	    
            args = """-c "set query latest" \
                      -c "select * from userTable where userId='%s'" \
                   """ % (self.uniqueId)
            output = self.getOutput(args)
            self.checkOneTuple(output)
            self.checkRightTuple(output)

	def testInsertInvalidColumn(self):
	    """
	    Check that an error is produced if you try to insert into an unknown column
	    """
            args = """-c "set producer continuous" \
                      -c "insert into userTable (UserID,aString,aReal,anInt) values ('%s', 'a', 1.2, 3)" \
                   """ % (self.uniqueId)
            output = self.getErrorOutput(args)
	    self.checkOutput(output, "ERROR")

	def testProducerPredicate(self):
	    """
	    Check that an error is produced when you try to insert a tuple which does
	    not match the predicate
	    """
	    args = """-c "set producer predicate userTable WHERE userId='wrong_%s'" \
	              -c "insert into userTable (userId,aString,aReal,anInt) values ('%s', 'a', 1.2, 3)" \
		   """ % (self.uniqueId, self.uniqueId)
	    output = self.getErrorOutput(args)
	    self.checkOutput(output, "ERROR")

	def testProducerPredicateWithoutWhere(self):
	    """
	    Check that you can specify a producer predicate without WHERE
	    """
	    args = """-c "set producer latest" \
		      -c "set producer predicate userTable userId='%s'" \
	              -c "insert into userTable (userId,aString,aReal,anInt) values ('%s', 'a', 1.2, 3)" \
		   """ % (self.uniqueId, self.uniqueId)
	    status = self.getStatus(args)
	    assert (status == 0)

	    time.sleep(30)
	    
	    args = """-c "set query latest" \
		      -c "select * from userTable where userId='%s'" \
		   """ % (self.uniqueId)
	    output = self.getOutput(args)
            self.checkOneTuple(output)
            self.checkRightTuple(output)

        def testBadInsertGoodInsert(self):
	    """
	    Test that when you insert a tuple which does not match the predicate
	    and then insert one which does you only get back the matching one
	    """
	    args = """-c "set producer continuous" \
	              -c "set timeout 10 minutes" \
	              -c "set producer predicate userTable WHERE aString='a'" \
	              -c "insert into userTable (userId,aString,aReal,anInt) values ('%s', 'b', 1.2, 3)" \
	              -c "insert into userTable (userId,aString,aReal,anInt) values ('%s', 'a', 1.2, 3)" \
		      -c "set query continuous" \
		      -c "set maxage 1 hours" \
		      -c "set timeout 5 seconds" \
                      -c "select * from userTable where userId='%s'" \
		   """ % (self.uniqueId, self.uniqueId, self.uniqueId)
	    output = self.getOutput(args)
            self.checkOneTuple(output)
            self.checkRightTuple(output)
		
class InformationTests(unittest.TestCase, TestBase):

	def setUp(self):
	    TestBase.__init__(self)
                		
	def suite(self):
	    return unittest.makeSuite(InformationTests, 'test')

	def testShowTables(self):
	    """
	    Check that 'show tables' works and includes 'userTable' in the list
	    """
	    output = self.getOutput('-c "show tables"')
	    self.checkOutput(output, "\|\s*userTable\s*\|")
	    
	def testDescribeTable(self):
	    """
	    Check that 'describe' works for userTable
	    """
	    output = self.getOutput('-c "describe userTable"')
	    self.checkOutput(output, "\|\s*userId\s*\|\s*VARCHAR\(\d+\)\s*\|\s*Yes\s*\|") 
	    self.checkOutput(output, "\|\s*aString\s*\|\s*VARCHAR\(\d+\)\s*\|\s*No\s*\|") 
	    self.checkOutput(output, "\|\s*aReal\s*\|\s*REAL\s*\|\s*No\s*\|") 
	    self.checkOutput(output, "\|\s*anInt\s*\|\s*INTEGER\s*\|\s*No\s*\|") 
	    self.checkOutput(output, "\|\s*MeasurementDate\s*\|\s*DATE\s*\|\s*No\s*\|") 
	    self.checkOutput(output, "\|\s*MeasurementTime\s*\|\s*TIME\s*\|\s*No\s*\|") 

        def testShowProducersOf(self):
	    """
	    Check that 'show producers of' produces something sensible
	    """
            self.continuousInsert()
            output = self.getOutput('-c "show producers of userTable"')
            self.checkOutput(output, "^\d+\s+producers for table userTable")
            
        def testShowProperties(self):
	    """
	    Check that 'show properties' works and includes the query timeout
	    """
            output = self.getOutput('-c "show properties"')
            self.checkOutput(output, ".*Query timeout")

	def testShowQuery(self):
	    """
	    Check that 'show query' returns the correct query type
	    """
	    output = self.getOutput('-c "set query history" -c "show query"')
            self.checkOutput(output, "Query type: history")
	    
	def testShowTimeout(self):
	    """
	    Check that 'show query' returns the correct timeout
	    """
	    output = self.getOutput('-c "set timeout 24 days" -c "show timeout"')
            self.checkOutput(output, "Timeout: 24.* days")
	    
	def testShowMaxAge(self):
	    """
	    Check that 'show maxage' returns the correct maximum age
	    """
	    output = self.getOutput('-c "set maxage 3 hours" -c "show maxage"')
            self.checkOutput(output, "Maximum age of tuples: 3.* hours")
	    
	def testShowProducer(self):
	    """
	    Check that 'show producer' returns the correct producer type
	    """
	    output = self.getOutput('-c "set producer latest" -c "show producer"')
            self.checkOutput(output, "Producer type: continuous latest")
	    
	def testShowProducerPredicateBadTable(self):
	    """
	    Check that 'show producer predicate' returns an error when table does not exist in the Schema
	    """
	    output = self.getErrorOutput("""-c "show producer predicate WrongTable" \
				    """)
            self.checkOutput(output, "ERROR")

	def testShowProducerPredicate(self):
	    """
	    Check that 'show producer predicate' returns the correct producer predicate
	    """
	    output = self.getOutput("""-c "set producer predicate userTable WHERE userId='fred'" \
	                               -c "show producer predicate userTable" \
				    """)
            self.checkOutput(output, "Producer predicate for 'userTable': WHERE userId='fred'")

	def testShowProducerPredicateNone(self):
	    """
	    Check that 'show producer predicate' gives the correct response when no predicate is defined
	    """
	    output = self.getOutput("""-c "set producer predicate userTable none" \
	                               -c "show producer predicate userTable" \
				    """)
            self.checkOutput(output, "No predicate defined for table 'userTable'")

	def testShowProducerHrp(self):
	    """
	    Check that 'show producer hrp' returns the correct history retention period
	    """
	    output = self.getOutput('-c "set producer hrp 5 minutes" -c "show producer historyretentionperiod"')
            self.checkOutput(output, "Set producer HRP to 5.* minutes")

	def testShowProducerLrp(self):
	    """
	    Check that 'show producer lrp' returns the correct latest retention period
	    """
	    output = self.getOutput('-c "set producer latestretentionperiod 6 minutes" -c "show producer lrp"')
            self.checkOutput(output, "Set producer LRP to 6.* minutes")

	def testShowSecondaryProducer(self):
	    """
	    Check that 'show secondaryproducer' returns the correct producer type
	    """
	    output = self.getOutput('-c "set secondaryproducer latest" -c "show secondaryproducer"')
            self.checkOutput(output, "Secondary producer type: continuous latest")

	def testShowSecondaryProducerHrp(self):
	    """
	    Check that 'show sp hrp' returns the correct history retention period
	    """
	    output = self.getOutput('-c "set sp hrp 5 minutes" -c "show sp historyretentionperiod"')
            self.checkOutput(output, "Set secondary producer HRP to 5.* minutes")
	    
class SetPropertiesTests(unittest.TestCase, TestBase):

        def setUp(self):
	    TestBase.__init__(self)
                		
	def suite(self):
	    return unittest.makeSuite(SetPropertiesTests, 'test')
	    
	# General set commands
	
	def testSetNoProperty(self):
	    output = self.getErrorOutput('-c "set"')
	    self.checkOutput(output, "ERROR")

	def testSetBadProperty(self):
	    output = self.getErrorOutput('-c "set giraffe"')
	    self.checkOutput(output, "ERROR")

        # set timeout
        
	def testSetTimeout(self):
            output = self.getOutput('-c "set timeout 0.7 days"')
            self.checkOutput(output, "Set timeout to 0.7.* days")

        def testSetTimeoutNoValue(self):
            output = self.getErrorOutput('-c "set timeout"')
            self.checkOutput(output, "ERROR")
             
        def testSetTimeoutDefaultUnits(self):
            output = self.getOutput('-c "set timeout 253"')
            self.checkOutput(output, "Set timeout to 253.* seconds")

        def testSetTimeoutTooManyArgs(self):
            output = self.getErrorOutput('-c "set timeout 34 hours and 23 minutes"')
            self.checkOutput(output, "ERROR")
	    
	# set maxage
	
	def testSetMaxAge(self):
            output = self.getOutput('-c "set maxage 2.3 hours"')
            self.checkOutput(output, "Set max age to 2.3.* hours")

	def testSetMaxAgeDefaultUnits(self):
            output = self.getOutput('-c "set maxage 175"')
            self.checkOutput(output, "Set max age to 175.* seconds")

        def testSetMaxAgeNoValue(self):
            output = self.getErrorOutput('-c "set maxage"')
            self.checkOutput(output, "ERROR")

        def testSetMaxAgeTooManyArgs(self):
            output = self.getErrorOutput('-c "set maxage 34 hours and 23 minutes"')
            self.checkOutput(output, "ERROR")
                       
        # set query
        
        def testSetQuery(self):
            output = self.getOutput('-c "set query latest"')
            self.checkOutput(output, "Set query type to latest")
            
        def testOnlyOneQueryType(self):
            output = self.getErrorOutput('-c "set query latest history"')
            self.checkOutput(output, "ERROR")

        def testSetQueryNoValue(self):
            output = self.getErrorOutput('-c "set query"')
            self.checkOutput(output, "ERROR")

	# set output format
        
        def testSetOutput(self):
            output = self.getOutput('-c "set output table"')
            self.checkOutput(output, "Set output format to 'table'")
            output = self.getOutput('-c "set output tsv"')
            self.checkOutput(output, "Set output format to 'tsv'")
            output = self.getOutput('-c "set output csv"')
            self.checkOutput(output, "Set output format to 'csv'")
            
        def testSetOutputBadFormat(self):
            output = self.getErrorOutput('-c "set output freddy"')
            self.checkOutput(output, "ERROR")

        def testSetOutputNoFormat(self):
            output = self.getErrorOutput('-c "set output"')
            self.checkOutput(output, "ERROR")
	    
        def testSetOutputOnlyOneFormat(self):
            output = self.getErrorOutput('-c "set output table csv"')
            self.checkOutput(output, "ERROR")
 
        # set producer
        
        def testSetProducer(self):
            output = self.getOutput('-c "set producer latest continuous"')
            self.checkOutput(output, "Producer type")
            self.checkOutput(output, ".*continuous")
            self.checkOutput(output, ".*latest")

        def testAllProducersContinuous(self):
            output = self.getOutput('-c "set producer latest"')
            self.checkOutput(output, "Producer type")
            self.checkOutput(output, ".*continuous")
            self.checkOutput(output, ".*latest")

        def testProducerQueryTypesOred(self):
            output = self.getOutput('-c "set producer continuous latest latest"')
            self.checkOutput(output, "Producer type")
            self.checkOutput(output, ".*continuous")
            self.checkOutput(output, ".*latest")

        def testSetProducerNoValue(self):
            output = self.getErrorOutput('-c "set producer"')
            self.checkOutput(output, "ERROR")

        # set producer hrp
        
        def testSetProducerHistoryRetentionPeriod(self):
            output = self.getOutput(' -c "set producer historyretentionperiod 5 hours"')
            self.checkOutput(output, "Set producer HRP to 5.* hours")

        def testSetProducerHistoryRetentionPeriodDefaultUnits(self):
            output = self.getOutput(' -c "set producer historyretentionperiod 987"')
            self.checkOutput(output, "Set producer HRP to 987.* seconds")

        def testSetProducerHistoryRetentionPeriodNoValue(self):
            output = self.getErrorOutput(' -c "set producer historyretentionperiod"')
            self.checkOutput(output, "ERROR")

        def testSetProducerHistoryRetentionPeriodTooManyArguments(self):
            output = self.getErrorOutput(' -c "set producer historyretentionperiod 6 hours seconds"')
            self.checkOutput(output, "ERROR")
            
        def testSetProducerHrp(self):
            output = self.getOutput(' -c "set producer hrp 5 hours"')
            self.checkOutput(output, "Set producer HRP to 5.* hours")

        def testSetProducerHrpDefaultUnits(self):
            output = self.getOutput(' -c "set producer hrp 987"')
            self.checkOutput(output, "Set producer HRP to 987.* seconds")

        def testSetProducerHrpNoValue(self):
            output = self.getErrorOutput(' -c "set producer hrp"')
            self.checkOutput(output, "ERROR")

        def testSetProducerHrpTooManyArguments(self):
            output = self.getErrorOutput(' -c "set producer hrp 6 hours seconds"')
            self.checkOutput(output, "ERROR")
            
        # set producer lrp
        
        def testSetProducerlatestRetentionPeriod(self):
            output = self.getOutput(' -c "set producer latestretentionperiod 5 hours"')
            self.checkOutput(output, "Set producer LRP to 5.* hours")

        def testSetProducerlatestRetentionPeriodDefaultUnits(self):
            output = self.getOutput(' -c "set producer latestretentionperiod 987"')
            self.checkOutput(output, "Set producer LRP to 987.* seconds")

        def testSetProducerlatestRetentionPeriodNoValue(self):
            output = self.getErrorOutput(' -c "set producer latestretentionperiod"')
            self.checkOutput(output, "ERROR")

        def testSetProducerlatestRetentionPeriodTooManyArguments(self):
            output = self.getErrorOutput(' -c "set producer latestretentionperiod 6 hours seconds"')
            self.checkOutput(output, "ERROR")
            
        def testSetProducerlrp(self):
            output = self.getOutput(' -c "set producer lrp 5 hours"')
            self.checkOutput(output, "Set producer LRP to 5.* hours")

        def testSetProducerlrpDefaultUnits(self):
            output = self.getOutput(' -c "set producer lrp 987"')
            self.checkOutput(output, "Set producer LRP to 987.* seconds")

        def testSetProducerlrpNoValue(self):
            output = self.getErrorOutput(' -c "set producer lrp"')
            self.checkOutput(output, "ERROR")

        def testSetProducerlrpTooManyArguments(self):
            output = self.getErrorOutput(' -c "set producer lrp 6 hours seconds"')
            self.checkOutput(output, "ERROR")
            
        # set producer predicate
        
        def testSetProducerPredicate(self):
	    """
	    Check that you can set a producer predicate
	    """
            output = self.getOutput(' -c "set producer predicate userTable WHERE userId=\'fred\'"')
            self.checkOutput(output, "Set producer predicate to 'WHERE userId='fred'' for table userTable")

        def testSetProducerPredicateNone(self):
	    """
	    Check that you can specify 'none' for a predicate
	    """
            output = self.getOutput(' -c "set producer predicate userTable none"')
            self.checkOutput(output, "Set producer predicate to None for table userTable")
        
        def testSetProducerPredicateNoTable(self):
	    """
	    Check that an error is produced when you don't specify a table name for the predicate
	    """
            output = self.getErrorOutput(' -c "set producer predicate"')
            self.checkOutput(output, "ERROR")

        def testSetProducerPredicateNoValue(self):
	    """
	    Check that an error is produced when you don't specify a value for the predicate
	    """
            output = self.getErrorOutput(' -c "set producer predicate userTable"')
            self.checkOutput(output, "ERROR")
        
        def testSetProducerPredicateInvalid(self):
	    """
	    Check that an error is produced when you specify an invalid predicate
	    """
            output = self.getErrorOutput(' -c "set producer predicate userTable WHERE userIdWrong=\'fred\'"')
            self.checkOutput(output, "ERROR")

        def testSetProducerPredicateInvalidValid(self):
	    """
	    Check that you can declare an invalid predicate followed by a valid one and the second one works
	    """
            output = self.getOutput(' -c "set producer predicate userTable WHERE userIdWrong=\'fred\'" -c "set producer predicate userTable WHERE userId=\'fred\'"')
            self.checkOutput(output, "Set producer predicate to")

        def testSetProducerPredicateBadTable(self):
	    """
	    Check that an error is produced when you specify a table which doesn't exist
	    """
            output = self.getErrorOutput(' -c "set producer predicate WrongTable WHERE userId=\'fred\'"')
            self.checkOutput(output, "ERROR")
        
        # set secondaryproducer
        
        def testSetSecondaryProducer(self):
            output = self.getOutput('-c "set secondaryproducer continuous latest history"')
            self.checkOutput(output, "Secondary producer type")
            self.checkOutput(output, ".*continuous")
            self.checkOutput(output, ".*latest")
            self.checkOutput(output, ".*history")
        
        def testAllSecondaryProducersContinuous(self):
            output = self.getOutput('-c "set secondaryproducer latest"')
            self.checkOutput(output, "Secondary producer type")
            self.checkOutput(output, ".*continuous")
            self.checkOutput(output, ".*latest")
            
        def testSetSecondaryProducerNoValue(self):
            output = self.getErrorOutput('-c "set secondaryproducer"')
            self.checkOutput(output, "ERROR")        

        def testSecondaryProducerQueryTypesOred(self):
            output = self.getOutput('-c "set secondaryproducer continuous latest latest"')
            self.checkOutput(output, "Secondary producer type")
            self.checkOutput(output, ".*continuous")
            self.checkOutput(output, ".*latest")
            
        # set secondaryproducer hrp
        
        def testSetsecondaryproducerHistoryRetentionPeriod(self):
            output = self.getOutput(' -c "set secondaryproducer historyretentionperiod 5 hours"')
            self.checkOutput(output, "Set secondary producer HRP to 5.* hours")

        def testSetsecondaryproducerHistoryRetentionPeriodDefaultUnits(self):
            output = self.getOutput(' -c "set secondaryproducer historyretentionperiod 987"')
            self.checkOutput(output, "Set secondary producer HRP to 987.* seconds")

        def testSetsecondaryproducerHistoryRetentionPeriodNoValue(self):
            output = self.getErrorOutput(' -c "set secondaryproducer historyretentionperiod"')
            self.checkOutput(output, "ERROR")

        def testSetsecondaryproducerHistoryRetentionPeriodTooManyArguments(self):
            output = self.getErrorOutput(' -c "set secondaryproducer historyretentionperiod 6 hours seconds"')
            self.checkOutput(output, "ERROR")
            
        def testSetsecondaryproducerHrp(self):
            output = self.getOutput(' -c "set secondaryproducer hrp 5 hours"')
            self.checkOutput(output, "Set secondary producer HRP to 5.* hours")

        def testSetsecondaryproducerHrpDefaultUnits(self):
            output = self.getOutput(' -c "set secondaryproducer hrp 987"')
            self.checkOutput(output, "Set secondary producer HRP to 987.* seconds")

        def testSetsecondaryproducerHrpNoValue(self):
            output = self.getErrorOutput(' -c "set secondaryproducer hrp"')
            self.checkOutput(output, "ERROR")

        def testSetsecondaryproducerHrpTooManyArguments(self):
            output = self.getErrorOutput(' -c "set secondaryproducer hrp 6 hours seconds"')
            self.checkOutput(output, "ERROR")
            
class AbbreviationsTests(unittest.TestCase, TestBase):

        def setUp(self):
	    TestBase.__init__(self)
                		
	def suite(self):
	    return unittest.makeSuite(AbbreviationsTests, 'test')
	    
        def testHRP(self):
            output = self.getOutput(' -c "set producer hrp 987"')
            self.checkOutput(output, "Set producer HRP to 987.* seconds")

        def testLRP(self):
            output = self.getOutput(' -c "set producer lrp 987"')
            self.checkOutput(output, "Set producer LRP to 987.* seconds")
        
        def testSP(self):
            output = self.getOutput(' -c "set sp latest" -c "sp userTable"')
            self.checkOutput(output, "Secondary producer type")     
            self.checkOutput(output, ".*latest")
            self.checkOutput(output, ".*userTable")     

        def testP(self):
            output = self.getOutput(' -c "set p latest"')
            self.checkOutput(output, "Producer type")     
            self.checkOutput(output, ".*latest")

	def testTrailingSemicolonIgnored(self):
	    output = self.getOutput(' -c "set p latest;"')
            self.checkOutput(output, "Producer type")     
            self.checkOutput(output, ".*latest")

class DirectedQueryTests(unittest.TestCase, TestBase):

        def setUp(self):
	    TestBase.__init__(self)
                		
	def suite(self):
	    return unittest.makeSuite(DirectedQueryTests, 'test')
	    
        def testUseMediator(self):
            output = self.getOutput(' -c "use mediator"')
            self.checkOutput(output, "Using mediator")
        
        def testUseMediatorArguments(self):
            output = self.getErrorOutput(' -c "use mediator please"')
            self.checkOutput(output, "ERROR")

        def testUseProducer(self):
            output = self.getOutput(' -c "use producer example.com 1234"')
            self.checkOutput(output, "Using producer")
            self.checkOutput(output, ".*example\.com")
            self.checkOutput(output, ".*1234")

        def testUseProducerNoArguments(self):
            output = self.getErrorOutput(' -c "use producer"')
            self.checkOutput(output, "ERROR")
 
        def testUseProducerOneArgument(self):
            output = self.getErrorOutput(' -c "use producer example.com"')
            self.checkOutput(output, "ERROR")

        def testUseProducerTooManyArguments(self):
            output = self.getErrorOutput(' -c "use producer example.com 1234 please"')
            self.checkOutput(output, "ERROR")

class HelpTests(unittest.TestCase, TestBase):

	def setUp(self):
		TestBase.__init__(self)

	def suite(self):
		return unittest.makeSuite(HelpTests, 'test')

	def testHelp(self):
	    status = self.getStatus(' -c "help"')
	    assert (status == 0)

	def testHelpCommand(self):
	    status = self.getStatus(' -c "help select"')
	    assert (status == 0)

	def testHelpShowProperties(self):
	    output = self.getOutput(' -c "help show"')
	    self.checkOutput(output, "show properties")
	    
	def testExamples(self):
	    status = self.getStatus(' -c "help examples"')
	    assert (status == 0)

        def testAllCommandsDocumented(self):
	    output = self.getOutput(' -c help')
            found = False
	    for line in output:
                if re.match("^Undocumented commands", line.strip()):
		    found = True
	    assert not found

class HistoryTests(unittest.TestCase, TestBase):

	def setUp(self):
		TestBase.__init__(self)

	def suite(self):
		return unittest.makeSuite(HistoryTests, 'test')

	def testShowHistory(self):
	    status = self.getStatus(' -c "show history"')
	    assert (status == 0)

	def testClearHistory(self):
	    status = self.getStatus(' -c "clear history"')
	    assert (status == 0)

	def testWriteHistory(self):
            tempName = tempfile.mktemp(".rgma")
	    try:
	        status = self.getStatus(' -c "show tables" -c "write history %s"' % tempName)
	        assert (status == 0)
                tempFile = open(tempName, "r")
	        lines = tempFile.readlines()
		tempFile.close()
	        assert(len(lines) == 1)
	        for line in lines:
	            assert(line == "show tables\n")
            finally:
                os.remove(tempName)
	    
	def testHistoryCommands(self):
	    output = self.getOutput(' -c "show tables" -c "show producers of userTable" -c "show history"')
	    self.checkOutput(output, "^show tables")
	    self.checkOutput(output, "^show producers of userTable")

	def testHistoryComments(self):
	    output = self.getOutput(' -c "show tables # a comment" -c "# another comment" -c "show history"')
	    self.checkOutput(output, "^show tables # a comment")
	    self.checkOutput(output, "^# another comment")

	def testHistoryIgnoresHistory(self):
	    output = self.getOutput(' -c "show tables" -c "show history"')
	    self.checkOutput(output, "show tables")
	    self.checkOutputNotPresent(output, "^history")

	def testHistoryIgnoresClear(self):
	    output = self.getOutput(' -c "clear history" -c "show tables" -c "show history"')
	    self.checkOutput(output, "show tables")
	    self.checkOutputNotPresent(output, "^clear")

	def testHistoryIgnoresHelp(self):
	    output = self.getOutput(' -c "show tables" -c "help" -c "show history"')
	    self.checkOutput(output, "show tables")
	    self.checkOutputNotPresent(output, "^help")
	    
class MiscTests(unittest.TestCase, TestBase):

	def setUp(self):
		TestBase.__init__(self)

	def suite(self):
		return unittest.makeSuite(MiscTests, 'test')

	def testBlankCommand(self):
		output = self.getOutput(' -c "set query latest" -c ""')
		matches = 0
		for line in output:
                    if re.match("Set query type to latest", line.strip()):
			matches += 1
		assert(matches == 1)

	def testComment(self):
		output = self.getOutput(' -c "# just a comment"')
		assert(len(output) == 0)
		
	def testCommandWithComment(self):
		output = self.getOutput(' -c "set timeout 0.1 seconds # just a comment"')
                self.checkOutput(output, "Set timeout to 0.1.* seconds")
		
	def testExitCodeCommandNotFound(self):
		status = self.getStatus(' -c "sillycommand"')
		assert(status == 1)
		
	def testExitCodeBadCommand(self):
		status = self.getStatus(' -c "set thingy 7"')
		assert(status == 1)

	def testRead(self):
            tempName = tempfile.mktemp(".rgma")
            try:
                tempFile = open(tempName, "w")
                tempFile.write("set timeout 0.1 seconds\n")
                tempFile.write("exit\n")
                tempFile.close()         
                output = self.getOutput('-c "read %s"' % tempName)
                self.checkOutput(output, "Set timeout to 0.1.* seconds")
                self.checkOutput(output, "Goodbye")
            finally:
                os.remove(tempName)

	def testReadFileWithEOF(self):
            tempName = tempfile.mktemp(".rgma")
            try:
                tempFile = open(tempName, "w")
                tempFile.write("set timeout 0.1 seconds\n")
                tempFile.write("EOF\n")
                tempFile.close()      
                output = self.getErrorOutput('-c "read %s"' % tempName)
                self.checkOutputNotPresent(output, "ERROR")
            finally:
                os.remove(tempName)

	def testEOF(self):
	    status = self.getStatus(' -c "EOF"')
	    assert (status == 0)

	def testCloseProducers(self):
            args = """-c "set producer continuous" \
                      -c "insert into userTable (userId, aString, aReal, anInt) values ('%s', 'a', 1.2, 3)" \
		      -c "close producers" \
		      -c "show properties" \
                   """ % self.uniqueId
            output = self.getOutput(args)
	    self.checkOutput(output, "")

	def testWriteResults(self):
	    """
	    Check the redirection of query results to a file
	    """
	    tempName = tempfile.mktemp(".rgma")
	    try:
	        self.continuousInsert()
		args = """-c "set query continuous" \
	                  -c "set timeout 5 seconds" \
			  -c "set maxage 1 hours" \
			  -c "write results %s" \
			  -c "select * from userTable where userId='%s'" \
		       """ % (tempName, self.uniqueId)
		status = self.getStatus(args)
		assert (status == 0)		
                tempFile = open(tempName, "r")
	        lines = tempFile.readlines()
		tempFile.close()
		self.checkOneTuple(lines)
		self.checkRightTuple(lines)
            finally:
                os.remove(tempName)

class CaseInsensitivityTests(unittest.TestCase, TestBase):

	def setUp(self):
		TestBase.__init__(self)

	def suite(self):
		return unittest.makeSuite(CaseInsensitivityTests, 'test')

	def testCaseInsensitiveSet(self):
		output = self.getOutput(' -c "Set Query Latest"')
	        self.checkOutput(output, "Set query type to latest")

	def testCaseInsensitiveHelp(self):
		output = self.getOutput(' -c "HeLp SeLecT"')
	        self.checkOutput(output, "select <columns>")

        def testCaseInsensitiveSelect(self):
            output = self.getOutput("""-c "set query latest" \
                                       -c "SelEct * fRom userTable whEre userId='%s'" \
				     """ % self.uniqueId)
            self.checkNoTuples(output)

class OutputFormatTests(unittest.TestCase, TestBase):

	def setUp(self):
		TestBase.__init__(self)

	def suite(self):
		return unittest.makeSuite(OutputFormatTests, 'test')

        def testTable(self):
	    """
	    Check table formatting 
	    """
            self.continuousInsert()
            args = """-c "set query continuous" \
                      -c "set timeout 5 seconds" \
                      -c "set maxage 1 hours" \
		      -c "set output table" \
                      -c "select * from userTable where userId='%s'" \
                   """ % self.uniqueId
            output = self.getOutput(args)
            regex = "\|\s*%s\s*\|\s*a\s*\|\s*1\.2\s*\|\s*3\s*\|\s*[\d-]+\s*\|\s*[\d:]+\s*\|\s*" % self.uniqueId
            self.checkOutput(output, regex)

        def testCsv(self):
	    """
	    Check comma-separated formatting 
	    """
            self.continuousInsert()
            args = """-c "set query continuous" \
                      -c "set timeout 5 seconds" \
                      -c "set maxage 1 hours" \
		      -c "set output csv" \
                      -c "select * from userTable where userId='%s'" \
                   """ % self.uniqueId
            output = self.getOutput(args)
            regex = "\s*%s,\s*a,\s*1\.2,\s*3,\s*[\d-]+,\s*[\d:]+\s*" % self.uniqueId
            self.checkOutput(output, regex)
	    
        def testTsv(self):
	    """
	    Check tab-separated formatting 
	    """
            self.continuousInsert()
            args = """-c "set query continuous" \
                      -c "set timeout 5 seconds" \
                      -c "set maxage 1 hours" \
		      -c "set output tsv" \
                      -c "select * from userTable where userId='%s'" \
                   """ % self.uniqueId
            output = self.getOutput(args)
            regex = "\s*%s\t\s*a\t\s*1\.2\t\s*3\t\s*[\d-]+\t\s*[\d:]+\s*" % self.uniqueId
            self.checkOutput(output, regex)

class SchemaTests(unittest.TestCase, TestBase):

	def setUp(self):
		TestBase.__init__(self)

	def suite(self):
		return unittest.makeSuite(SchemaTests, 'test')

	def testCreateDrop(self):
		create = "create table temp_%s (userId VARCHAR(255) PRIMARY KEY,\
		          aString VARCHAR(255), aReal REAL, anInt INTEGER)" % self.uniqueId
		args = '-c "%s"'% create
		status = self.getStatus(args)
		assert status == 0
		args = '-c "drop table temp_%s"' % self.uniqueId
		status = self.getStatus(args)
		assert status == 0
		
if __name__ == "__main__":
	unittest.main()
